import pymouse
import sys



def PyMouse():
    '''
    wrapper, turn off print coming from Xlib.ext.randr
    '''
    #Quick, turn off printing!
    class dummyStream:
        ''' dummyStream behaves like a stream but does nothing. '''
        def __init__(self): pass
        def write(self, data): pass
        def read(self, data): pass
        def flush(self): pass
        def close(self): pass
    # Copy old print deals
    old = sys.stdout
    # redirect all print deals
    sys.stdout = dummyStream()
    
    #call your verbose function here
    m = pymouse.PyMouse()
    
    #Turn printing back on!
    sys.stdout = old
    
    return m  



from easyprocess import EasyProcess
from pyscreenshot import grab
from discogui.imgutil import autocrop, focus_wnd
from discogui.sendkeys import send_key_list
from pyvirtualdisplay import Display
import time


def main():
    screen = Display()
    screen.start()
    
    gcalctool = EasyProcess('gcalctool').start()
    
    # wait for displaying the window
    time.sleep(0.2)
    
    focus_wnd()
    
    send_key_list(['2', '*', '2', '=', '\n'])
    
    img = autocrop(grab())
    
    gcalctool.stop()
    screen.stop()
    
    # after stopping screen!
    img.show()

if __name__=='__main__':
    main()


# -*- Import: -*-
from paver.easy import *
from paver.setuputils import setup
from setuptools import find_packages

try:
    # Optional tasks, only needed for development
    # -*- Optional import: -*-
    import paver.doctools
    import paver.virtual
    import paver.misctasks
    ALL_TASKS_LOADED = True
except ImportError, e:
    info("some tasks could not not be imported.")
    debug(str(e))
    ALL_TASKS_LOADED = False

NAME = 'discogui'
PACKAGE = 'discogui'
URL = 'https://github.com/ponty/discogui'
DESCRIPTION = 'GUI discovery'

__version__ = None
py = path('.') / PACKAGE / '__init__.py'
for line in open(py).readlines():
    if '__version__' in line:
        exec line
        break
assert __version__    
version = __version__

classifiers = [
    # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
    "License :: OSI Approved :: BSD License",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    ]

install_requires = [
    # -*- Install requires: -*-
    'setuptools',
    'path.py',
    'PyMouse',
    'PyScreenshot',
    'easyprocess',
    'PIL',
    ]

entry_points = """
    # -*- Entry points: -*-
    """

# compatible with distutils of python 2.3+ or later
setup(
    name=NAME,
    version=version,
    description=DESCRIPTION,
    long_description=open('README.rst', 'r').read(),
    classifiers=classifiers,
    keywords='GUI',
    author='ponty',
    #author_email='zy@gmail.com',
    url=URL,
    license='BSD',
    packages=find_packages(exclude=['bootstrap', 'pavement', ]),
    include_package_data=True,
    test_suite='nose.collector',
    zip_safe=False,
    install_requires=install_requires,
    entry_points=entry_points,
    )

options(
    # -*- Paver options: -*-
    minilib=Bunch(
        extra_files=[
            # -*- Minilib extra files: -*-
            ]
        ),
    sphinx=Bunch(
        docroot='docs',
        builddir="_build",
        sourcedir=""
        ),
    virtualenv=Bunch(
        packages_to_install=[
            # -*- Virtualenv packages to install: -*-
            "nose",
            "Sphinx>=0.6b1",
            "pkginfo",
            "virtualenv"],
        dest_dir='./virtual-env/',
        install_paver=True,
        script_name='bootstrap.py',
        paver_command_line=None
        ),
    )

options.setup.package_data = paver.setuputils.find_package_data(
    PACKAGE, package=PACKAGE, only_in_packages=False)

if ALL_TASKS_LOADED:
    @task
    @needs('generate_setup', 'minilib', 'setuptools.command.sdist')
    def sdist():
        """Overrides sdist to make sure that our setup.py is generated."""

@task
def pychecker():
    sh('pychecker --stdlib --only --limit 100 {package}/ '.format(package=PACKAGE))

@task
def findimports():
    '''list external imports'''
    sh('findimports {package} |grep -v ":"|grep -v {package}|sort|uniq'.format(package=PACKAGE))

@task
def pyflakes():
    sh('pyflakes {package}'.format(package=PACKAGE))

@task
def nose():
    sh('nosetests --with-xunit --verbose')

@task
def sloccount():
    sh('sloccount --wide --details {package} tests > sloccount.sc'.format(package=PACKAGE))

@task
def clean():
    root=path(__file__).dirname().abspath()
    ls=[]
    dls=[]
    ls+=root.walkfiles('*.pyc')
    ls+=root.walkfiles('*.html')
    ls+=root.walkfiles('*.zip')
    ls+=root.walkfiles('*.css')
    ls+=root.walkfiles('*.png')
    ls+=root.walkfiles('*.doctree')
    ls+=root.walkfiles('*.pickle')

    dls+=[root / 'dist']
    dls+=root.listdir('*.egg-info')

    for x in ls:
        x.remove()
    for x in dls:
        x.rmtree()

@task
@needs('sloccount', 'html', 'sdist', 'nose')
def hudson():
    pass





'''
1. start zenity Yes/No dialog on Xvfb
2. discover buttons using :mod:`discogui.buttons` module
3. print rectangles
4. draw red rectangles on screenshot
'''
from easyprocess import EasyProcess
from pyscreenshot import grab
from discogui.buttons import discover_buttons
from discogui.draw import draw_indexed_rect_list
from discogui.imgutil import autocrop
from pyvirtualdisplay import Display

def main():
    screen = Display().start()
    zenity= EasyProcess('zenity --question').start().sleep(1)
    
    img = grab()
    rectangles = discover_buttons()
    print rectangles
    
    zenity.stop()
    screen.stop()
    
    img = draw_indexed_rect_list(img, rectangles)
    img = autocrop(img)
    
    # display results
    img.show()

if __name__=='__main__':
    main()    


# -*- coding: utf-8 -*-

from path import path
import logging
import sphinx
import sys

project='discogui'
author='ponty'
copyright = '2011, ponty'
PACKAGE = 'discogui'

__version__ = None
py = path('..') / PACKAGE / '__init__.py'
for line in open(py).readlines():
    if '__version__' in line:
        exec line
        break
assert __version__    
release = __version__

#logging.basicConfig(level=logging.DEBUG)

# Extension
extensions = [
     # -*-Extensions: -*-
     'sphinx.ext.autodoc',
     'sphinxcontrib.programoutput',
     'sphinxcontrib.programscreenshot',
     #'sphinx.ext.graphviz',
     #'sphinx.ext.autosummary',
    ]
#intersphinx_mapping = {'http://docs.python.org/': None}

# Source
master_doc = 'index'
templates_path = ['_templates']
source_suffix = '.rst'
exclude_trees = []
pygments_style = 'sphinx'

# html build settings
html_theme = 'default'
html_static_path = ['_static']

# htmlhelp settings
htmlhelp_basename = '%sdoc' % project

# latex build settings
latex_documents = [
    ('index', '%s.tex' % project, u'%s Documentation' % project,
    author, 'manual'),
]



from logging import DEBUG
from path import path
from tempfile import mkdtemp, gettempdir
from discogui.draw import draw_indexed_rect_list
import logging

log = logging.getLogger(__name__)

img_dir = None
img_ind = 0
CROP_RECT = None

def set_crop_rect(rct):
    global CROP_RECT
    CROP_RECT = rct
    
def img_log(im, text):
    '''
    save the image in a temp folder for debugging.
    '''
    if not log.isEnabledFor(DEBUG):
        return
    global img_dir
    global img_ind
    if not img_dir:
        root = path(gettempdir()) / 'img_log'
        if not root.exists():
            root.makedirs()
        img_dir = path(mkdtemp(prefix='img_debug_', suffix='', dir=root))
    if CROP_RECT:
        im = im.crop(CROP_RECT)
    fname = str(img_dir / str(img_ind) + '_' + text + '.png')
    im.save(fname)
    log.debug('image was saved:' + fname)
    img_ind += 1

def img_log_rects(img, rcts, text):
    '''
    1. draw rectangles on image
    2. save image
    '''
    if not log.isEnabledFor(DEBUG):
        return
    img2 = draw_indexed_rect_list(img, rcts)
    img_log(img2, text + '_with_rects')



'''
Try to find buttons on the window.

1. get controls by :mod:`discogui.taborder`
2. test controls with :mod:`discogui.hover`, if control is active 
        -> it is identified as a button
3. no controls by tab order -> only 1 button on window 
        -> test whole window by :mod:`discogui.hover`  

.. note::
     It does not work if the GUI changes during the scan.
     (e.g. blinking cursor)

'''

from discogui import  hover
from discogui.imglog import  img_log_rects
from discogui.imgutil import  focus_wnd
from discogui.taborder import tab_rectangles
import logging

log = logging.getLogger(__name__)

def discover_buttons(grid=30):
    '''
    try to get buttons by tab order 
    '''
    tab_ls = tab_rectangles()
    
    if not len(tab_ls):
        log.debug('no tab order on window -> full hover test')
        # slow, no tab order
        return hover.active_rectangles(grid=grid)

    img_orig = focus_wnd()

    ls = []
    for x in tab_ls:
        bbox = hover.is_point_active(img_orig, x.center)   
        if bbox:
            ls += [x]
    
    img_log_rects(img_orig, ls, 'img_orig')
    return ls




'''
simple xdotool wrapper.

There is no easy linux Sendkeys modul, 
I have found this tools:
 * `SendKeys<http://www.rutherfurd.net/python/sendkeys/>`
       - windows only
 * xmacro, xdotool, xaut
       - no python interface
 * `xautomation<http://sourceforge.net/projects/xautomation/>`
          - Python library to automate X11
          - hard to install
 * `pykey<http://www.shallowsky.com/software/crikey/pykey-0.1>`
          - no setup      

This  module is used until crossplatform Sendkey modul is developed.
'''    

from easyprocess import EasyProcess
import time

EasyProcess('xdotool --version').check()

MODIFIERS = [
  "alt",
  "ctrl",
  "control",
  "meta",
  "super",
  "shift",
]

tmp = dict([
  ("Return", '\n',),
  ("ampersand", '&',),
  ("apostrophe", '\'',),
  ("asciicircum", '^',),
  ("asciitilde", '~',),
  ("asterisk", '*',),
  ("at", '@',),
  ("backslash", '\\',),
  ("bar", '|',),
  ("braceleft", '(',),
  ("braceright", ')',),
  ("bracketleft", '[',),
  ("bracketright", ']',),
  ("colon", ':',),
  ("comma", ',',),
  ("dollar", '$',),
  ("equal", '=',),
  ("exclam", '!',),
  ("grave", '`',),
  ("greater", '>',),
  ("less", '<',),
  ("minus", '-',),
  ("numbersign", '#',),
  ("parenleft", '(',),
  ("parenright", ')',),
  ("percent", '%',),
  ("period", '.',),
  ("plus", '+',),
  ("question", '?',),
  ("quotedbl", '"',),
  ("semicolon", ';',),
  ("slash", '/',),
  ("space", ' ',),
  ("Tab", '\t',),
  ("underscore", '_',),
  ("Escape", '\x1b',),
        ])

#inverted dict
XDOTOOL_KEY_MAP = dict([[v, k] for k, v in tmp.items()])

def send_key(key):
    '''
    send one key
    '''    
    x=key
    modifier = ''
    for m in MODIFIERS:
        if x.startswith(m):
            x = x.replace(m + '+', '')
            modifier = m + '+'
            break
    x = XDOTOOL_KEY_MAP.get(x, x)
    x = modifier + x
    EasyProcess(cmd=["xdotool", 'key', x ]).call()

def send_key_list(keys, pause=0.05):
    '''
    send more keys
    
    :param pause: The number of seconds to wait between sending each key or key combination. (float)
    '''    
    for x in keys:
        time.sleep(pause)
        send_key(x)

def _test():
    send_key_list(['alt+f'])

if __name__ == '__main__':
    _test()  
   



'''
1. start gnumeric on Xvfb with low ersolution
2. discover buttons using :mod:`discogui.hover` module
3. print rectangles
4. draw red rectangles on screenshot
'''
from discogui.draw import draw_indexed_rect_list
from discogui.hover import active_rectangles
from discogui.imgutil import autocrop
from easyprocess import EasyProcess
from pyscreenshot import grab
from pyvirtualdisplay import Display

def main():
    screen = Display(size=(640,480)).start()
    gnumeric= EasyProcess('gnumeric').start().sleep(2)
    
    img = grab()
    rectangles = active_rectangles()
    print rectangles
    
    gnumeric.stop()
    screen.stop()
    
    img = draw_indexed_rect_list(img, rectangles)
    img = autocrop(img)
    
    # display results
    img.show()

if __name__=='__main__':
    main()


'''
Copy of Rect5 (http://pypi.python.org/pypi/Rect/5)

changes:
- top and bottom are exchanged
- int instead float
- new methods starting at __getitem__
'''

class ScreenRect(object):
    """
    The ScreenRect class is used for storing and manipulating rectangular areas.

    It has left, top, width and height attributes, which are automatically
    changed by assignment to the right, bottom, bottomleft, bottomright, topleft, 
    topright or center properties.

    Rects can be added to greater a greater containing rectangle, or a 
    Rect.union classmethod is available to sum a list of Rect objects.

    The collidepoint and intersects methods are used for collision testing.

    Eg:
    >> Rect((0,0,10,10)).collidepoint((2,2))
    >> True
    >> Rect((0,0,10,10)).collidepoint((20,20))
    >> False

    This Rect class is different to the Pygame Rect class, in that is stores
    coordinates internally as ints, and uses a left-handed coordinate
    system.


    """
    def __init__(self, *args, **kwargs):
        """
        xywh must be a 2 or 4 tuple or a rect instance.
        
        (left, top, width, height) or (width, height)
        """
        from_size = kwargs.get('from_size', False)
        if len(args) == 1:
            xywh = args[0]
        else:
            xywh = args
        if isinstance(xywh, ScreenRect):
            self.left = xywh.left
            self.top = xywh.top
            self.width = xywh.width
            self.height = xywh.height
        else:
            if len(xywh) == 4:
                if from_size:
                    self.left, self.top, self.width, self.height = (int(i) for i in xywh)
                else:
                    x = list(int(i) for i in xywh)
                    self.left, self.top, self.width, self.height = (x[0], x[1], x[2] - x[0], x[3] - x[1])
            elif len(xywh) == 2:
                self.left, self.top, self.width, self.height = (0, 0) + tuple(int(i) for i in xywh)


    def __repr__(self):
        return "%s((%s,%s,%s,%s))" % (self.__class__.__name__, self.left, self.top, self.right, self.bottom)

    def __iter__(self):
        return (i for i in (self.left, self.top, self.right, self.bottom))

    def set_bottom(self, s):
        self.top = s - self.height
    def get_bottom(self):
        return self.top + self.height
    bottom = property(get_bottom, set_bottom)

    def set_right(self, s):
        self.left = s - self.width
    def get_right(self):
        return self.left + self.width
    right = property(get_right, set_right)

    def set_center(self, xy):
        self.left = xy[0] - (self.width * 0.5)
        self.top = xy[1] - (self.height * 0.5)
    def get_center(self):
        return self.left + (self.width * 0.5), self.top + (self.height * 0.5)
    center = property(get_center, set_center)

    def set_bottomleft(self, xy):
        self.left = xy[0]
        self.bottom = xy[1]
    def get_bottomleft(self):
        return self.left, self.bottom
    bottomleft = property(get_bottomleft, set_bottomleft)

    def set_bottomright(self, xy):
        self.right = xy[0]
        self.bottom = xy[1]
    def get_bottomright(self):
        return self.right, self.bottom
    bottomright = property(get_bottomright, set_bottomright)

    def set_topright(self, xy):
        self.right = xy[0]
        self.top = xy[1]
    def get_topright(self):
        return self.right, self.top
    topright = property(get_topright, set_topright)

    def set_topleft(self, xy):
        self.left = xy[0]
        self.top = xy[1]
    def get_topleft(self):
        return self.left, self.top
    topleft = property(get_topleft, set_topleft)

    def __add__(self, other):
        left = min(self.left, other.left)
        top = min(self.top, other.top)
        right = max(self.right, other.right)
        bottom = max(self.bottom, other.bottom)
        return ScreenRect((left, top, right - left, bottom - top))

    def add(self, other):
        """
        Add another rect to this rect, expanding as needed.
        """
        self.left = min(self.left, other.left)
        self.top = min(self.top, other.top)
        self.width = max(self.right, other.right) - self.left
        self.height = max(self.bottom, other.bottom) - self.top

    @classmethod
    def sum(cls, others):
        """
        Return a rect which covers all rects in others.
        """
        others = list(others)
        left, top, width, height = others.pop()
        right = left + width
        bottom = top + height
        for other in others:
            if other.left < left: left = other.left
            if other.top < top: top = other.top
            if other.right > right: right = other.right
            if other.bottom > bottom: bottom = other.bottom
        return cls((left, top, right - left, bottom - top))

    def collidepoint(self, xy):
        """
        Test if a point intersects with this rect.
        """
        x, y = xy
        return x >= self.left and x <= (self.left + self.width) and y >= self.top and y <= (self.top + self.height)

    def intersects(self, other):
        """
        Test if a rect intersects with this rect.
        """
        if self.left > other.left + other.width: return False
        if self.top > other.top + other.height: return False
        if self.left + self.width < other.left: return False
        if self.top + self.height < other.top: return False
        return True 

    def get_area(self):
        return self.width * self.height
    area = property(get_area)

    def intersection(self, other):
        """
        Return the intersection of this rect and other rect.
        Return None if no intersection.
        """
        left = max((self.left, other.left))
        top = max((self.top, other.top))
        right = min((self.left + self.width, other.left + other.width))
        bottom = min((self.top + self.height, other.top + other.height))
        if left > right or bottom < top: return None
        return ScreenRect((left, top, right - left, bottom - top))

    def contains(self, other):
        """
        Return True if other contains self
        """
        if other.left >= self.left and other.right <= self.right:
            if other.bottom <= self.bottom and other.top >= self.top:
                return True
        return False

    def fits(self, other):
        return self.width >= other.width and self.height >= other.height

    ####################################################################
    # new methods
    ####################################################################
    def __getitem__(self, i):
        x = (self.left, self.top, self.right, self.bottom)
        return x[i]

    @classmethod
    def from_points(cls, obj):
        """
        create from points:
        (left,top,right,bottom)
        """
        return cls((obj[0], obj[1], obj[2] - obj[0], obj[3] - obj[1]))

    def __len__(self):
        return 4
    
    def move(self, vector):
        self.left += vector[0]
        self.top += vector[1]
    
    def __eq__(self, other):
        x = (self.left, self.top, self.right, self.bottom)
        y = (other.left, other.top, other.right, other.bottom)        
        return all(a == b for a, b in zip(x, y))
    
    def point_inside(self, point):
        """
        Return True if self contains point
        """
        if point[0] >= self.left and point[0] <= self.right:
            if point[1] <= self.bottom and point[1] >= self.top:
                return True
        return False

    def add_border(self, border=1):
        """
        Return Rect
        """
        new = ScreenRect(self)
        new.left -= border
        new.top -= border
        new.width += 2 * border
        new.height += 2 * border
        return new

    
    
    
    
    
    
    



'''
This method moves the mouse over a grid, and compare
the image with the original and finds active regions.

.. note::
     It does not work if the GUI changes during the scan.
     (e.g. blinking cursor)

It can be slow if grid is small or the window is large.

'''

from discogui.imglog import img_log, img_log_rects
from discogui.imgutil import getbbox, focus_wnd
from discogui.mouse import PyMouse
import ImageChops
import logging
import pyscreenshot


log = logging.getLogger(__name__)
#log = logging

def is_point_active(img_orig, point, mouse=None):
    '''
    Check point on screen:
    1. move mouse over specified point
    2. move mouse to (0,0)
    
    If screen changes, then returns the bounding rectangle.

    Does not work if other parts of screen are changing (e.g. blinking cursor)
    
    :rtype: rectangle or None
    '''

    if not mouse:
        mouse = PyMouse()

    
    log.debug('point:' + str(point))

    mouse.move(point[0], point[1])
    img_hover = pyscreenshot.grab()
    mouse.move(0, 0)

    #img_log(img_hover, 'img_hover')
    img_log_rects(img_hover, [point + point], 'img_hover')

    img_diff = ImageChops.difference(img_orig, img_hover)
    # enhance color for debug
    img_diff = img_diff.point(lambda x: 255 * bool(x)) 
    img_log(img_diff, 'img_diff')
    
    bbox = getbbox(img_diff)
    if bbox:
        #assert bbox.point_inside(point), (bbox, point)
        if not bbox.point_inside(point):
            log.debug('point%s outside box%s' %(point, bbox))
        return bbox

def active_rectangles(grid=30):
    '''
    Return active rectangles found on the specified grid.
    Does not work if other parts of screen are changing (e.g. blinking cursor)
    
    :rtype: rectangles list
    '''
    mouse = PyMouse()
    
    img_orig = focus_wnd()
    rct_wnd = getbbox(img_orig)

    ls = []
    for y in xrange(rct_wnd.top + grid / 2, rct_wnd.bottom, grid - grid / 2):
        for x in xrange(rct_wnd.left + grid / 2, rct_wnd.right - grid / 2, grid):
            bbox = is_point_active(img_orig, (x, y), mouse=mouse)
            if bbox and bbox not in ls:
                ls.append(bbox)

    img_log_rects(img_orig, ls, 'img_orig')
    return ls
    
    



'''
This module search for GUI controls by sending TAB button events
and comparing the image with the original.

.. note::
     It does not work if the GUI changes during the scan.
     (e.g. blinking cursor)

'''

from discogui.imglog import img_log, img_log_rects
from discogui.imgutil import getbbox, focus_wnd
from discogui.screenrect import ScreenRect
from discogui.sendkeys import send_key
import ImageChops
import ImageFilter
import ImageStat
import logging
import pyscreenshot


log = logging.getLogger(__name__)

   
def darker(im1, im2, box):
    '''
    im1 is darker than im2 in box -> 0 
    im2 is darker than im1 in box -> 1 
    '''
    # add border, because box is an 'inside box', crop needs 'outside box'
    box = box.add_border(1)

    stat1 = ImageStat.Stat(im1.crop(box))
    stat2 = ImageStat.Stat(im2.crop(box))
    sum1 = sum(stat1.sum)
    sum2 = sum(stat2.sum)
    assert sum1 != sum2, box
    return 0 if sum1 > sum2 else 1


def tab_rectangles():
    '''
    Return rectangles found by sending TAB button events.
    
    Does not work if other parts of screen are changing (e.g. blinking cursor)

    :rtype: rectangles list
    '''
    ls = []
    
    img_orig = focus_wnd()

    im1 = img_orig

    while 1:
        send_key('\t')
        im2 = pyscreenshot.grab()

        img_log(im1, 'im1')
        img_log(im2, 'im2')
        
        boxes = tab_rect_pair(im1, im2)
        if not boxes:
            return []
        if len(ls):
            # TODO: implement almost_equal()
            assert boxes[0] == ls[-1]
            if boxes[1] in ls:
                break
            ls += [boxes[1]]
        else:
            #first
            ls += boxes
        im1 = im2
        
    img_log_rects(img_orig, ls, 'img_orig')
    return ls

def tab_rect_pair(img_orig, im_next):
    '''
    '''
    img_diff = ImageChops.difference(img_orig, im_next)
    img_log(img_diff, 'img_diff')
    
    # can be dotted -> filter + enhance color
    img_diff_filtered = img_diff.filter(ImageFilter.MaxFilter(5))
    img_diff_filtered = img_diff_filtered.point(lambda x: 255 * bool(x)) 
    img_log(img_diff_filtered, 'img_diff_filtered')

    bbox = getbbox(img_diff)
    if not bbox:
        return None

    
    def check_edges(horiz):
        if horiz:
            r1 = bbox.left
            r2 = bbox.right
        else:
            r1 = bbox.top
            r2 = bbox.bottom
        ls = []
        for c in xrange(int(r1), int(r2)):
            if horiz:
                p1 = (c, bbox.top)
                p2 = (c, bbox.bottom)
            else:
                p1 = (bbox.left, c)
                p2 = (bbox.right, c)
            color1 = sum(img_diff_filtered.getpixel(p1))
            color2 = sum(img_diff_filtered.getpixel(p2))
            ls += [int(bool(color1 + color2))] 
        
        if not 0 in ls:
            log.debug('split pos not found')
            return
        i = ls.index(0)
        if i == 0: 
            ls.reverse()
            i = ls.index(0)
            i = len(ls) - i - 1
        pos = i + r1
        log.debug('split pos found:%s' % pos)
        if horiz:
            rsegment1 = ScreenRect(0, 0, pos, img_orig.size[1])
            rsegment2 = ScreenRect(pos, 0, img_orig.size[0], img_orig.size[1])
        else:
            rsegment1 = ScreenRect(0, 0, img_orig.size[0], pos)
            rsegment2 = ScreenRect(0, pos, img_orig.size[0], img_orig.size[1])
        box1 = getbbox(img_diff.crop(rsegment1))
        box1.move(rsegment1.topleft)
        
        box2 = getbbox(img_diff.crop(rsegment2))
        box2.move(rsegment2.topleft)
                
        return   box1, box2
    r = check_edges(0)
    if r is None:
        r = check_edges(1)
    box1, box2 = r
    d1 = darker(img_orig, im_next, box1)
    d2 = darker(img_orig, im_next, box2)
    assert d1 != d2, ((d1, box1), (d2, box2))
    if d1 == 1:
        boxes = (box1, box2)
    else:
        boxes = (box2, box1)
    return boxes



'''
drawing rectangles for debugging
'''


from discogui.screenrect import ScreenRect
import Image
import ImageDraw

def draw_textrect(im, rect, text=None, color='red'):
    '''
    draw rectangle and text in center
    '''
    im = im.copy()
    draw = ImageDraw.Draw(im)
    draw.rectangle(rect, outline=color, fill=None)
    if text:
        rtext = ScreenRect(draw.textsize(text))
        rtext.center = ScreenRect(rect).center
        draw.text(rtext.topleft, text, fill=color)
    return im
        
def draw_indexed_rect_list(im, lsrect, color='red'):
    '''
    draw rectangles and index in center
    '''
    index = 0
    for x in lsrect:
        im = draw_textrect(im, x, str(index), color=color)
        index += 1
    return im
    
def _test(): 
    im = Image.new('RGB', (522, 222), 'grey')   
    x = draw_indexed_rect_list(im,
        [(1, 111, 111, 61), (221, 1, 111, 61), ] 
        ,)
    x.show()
    
if __name__ == '__main__':
    _test()
    
    
    



'''
1. start zenity Yes/No dialog on Xvfb
2. discover buttons using :mod:`discogui.buttons` module
3. print rectangles
'''
from discogui.buttons import discover_buttons
from easyprocess import EasyProcess
from pyvirtualdisplay import Display

def main():
    buttons = Display().wrap(
                    EasyProcess('zenity --question').wrap(
                                          discover_buttons, delay=1))
    print buttons()
    

if __name__ == '__main__':
    main()


'''
1. start zenity Yes/No dialog on Xvfb
2. discover buttons using :mod:`discogui.buttons` module
3. click first button, print return code
4. click second button, print return code
'''
from discogui.buttons import discover_buttons
from discogui.mouse import PyMouse
from easyprocess import EasyProcess
from pyvirtualdisplay import Display

def click_button_get_return_code(which_button):
    zenity = EasyProcess('zenity --question').start().sleep(1)
    rectangles = discover_buttons()
    PyMouse().click(*rectangles[which_button].center)
    return zenity.wait().return_code

def main():
    screen = Display().start()
    print click_button_get_return_code(0)
    print click_button_get_return_code(1)
    screen.stop()

if __name__=='__main__':
    main()


from discogui import imglog
from discogui.mouse import PyMouse
from discogui.screenrect import ScreenRect
import pyscreenshot

def getbbox(img, outside=False):
    '''
    Get bounding box.
    PIL getbbox() is not symmetrical on left/right.
    
    outside=0 -> box contains edges
    outside=1 -> box does not contain edges
    
    :param img: PIL image
    :param outside: bool
    :rtype: ScreenRect
    '''
    bbox = img.getbbox()
    if bbox:
        bbox = ScreenRect(bbox)
        if outside:
            bbox.left -= 1
            bbox.top -= 1
        else:
            bbox.right += 1
            bbox.bottom += 1
        return bbox

def autocrop(img, outside=False):
    box = getbbox(img, outside=outside)
    if box:
        img = img.crop(box)
    return img
        
class EmptyScreenException(Exception):
    pass

def focus_wnd():
    '''
    move mouse over window to get focus
    '''
    rct_wnd = getbbox(pyscreenshot.grab())
    if not rct_wnd:
        raise EmptyScreenException('Empty screen!')

    imglog.set_crop_rect(rct_wnd)

    # init window focus
    mouse = PyMouse()
    mouse.move(*rct_wnd.topleft)

    img_orig = pyscreenshot.grab()
    imglog.img_log(img_orig, 'img_orig')
    return img_orig
    
    
